"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const query_string_1 = require("query-string");
const wrapDisplayName_1 = __importDefault(require("recompose/wrapDisplayName"));
function stringToBoolean(value) {
    return value === 'true' ? true : false;
}
/**
 * Takes an query params object as created by the query-string's parse
 * function, and a definition for the defaultQueryParams, and tries
 * to convert from 'string' to a more concrete type.
 *
 * query-string's parse always returns strings, since we know the
 * default value for a query param, we can try to convert those
 * strings to the type of the default value for that query param.
 *
 * When an array is encountered it converts all values to the type
 * of the first item in the default values array. If the default
 * values array is empty, it defaults to only strings.
 *
 * @param {Object} queryParams
 * @param {Object} defaultQueryParams
 */
function convertQueryParamsToConcreteType(queryParams, defaultQueryParams, componentName) {
    const typedQueryParams = {};
    Object.keys(queryParams).forEach((key) => {
        const defaultValue = defaultQueryParams[key];
        if (defaultValue === undefined) {
            console.warn(`mad-react-url: no default query param defined for "${key}" for component: "${componentName}".`);
        }
        const actualValue = queryParams[key];
        if (actualValue instanceof Array) {
            const first = defaultValue[0];
            if (first === undefined) {
                typedQueryParams[key] = actualValue;
            }
            else {
                switch (typeof first) {
                    case 'number':
                        typedQueryParams[key] = actualValue.map(s => parseFloat(s));
                        break;
                    case 'boolean':
                        typedQueryParams[key] = actualValue.map(stringToBoolean);
                        break;
                    default:
                        typedQueryParams[key] = actualValue;
                        break;
                }
            }
        }
        else {
            // actualValue should be a string here.
            switch (typeof defaultValue) {
                case 'number':
                    typedQueryParams[key] = parseFloat(actualValue);
                    break;
                case 'boolean':
                    typedQueryParams[key] = stringToBoolean(actualValue);
                    break;
                default:
                    typedQueryParams[key] = actualValue;
            }
        }
    });
    return typedQueryParams;
}
exports.convertQueryParamsToConcreteType = convertQueryParamsToConcreteType;
/**
 * A HoC which augments the props with the queryParams from the
 * react-router-dom's `Location.search` string as an actual object.
 *
 * Requires defaultQueryParameters to fill in the blanks when the
 * query parameters are not in the url. This way you are guaranteed
 * to always have values for your query parameters.
 *
 * It also uses the defaultQueryParameters to convert the query
 * parameters (which all come out as strings) to the type of
 * the defaultQueryParameter.
 *
 * @example
 * withQueryParams(App, { page: 1, search: '', showAll: true })
 *
 * This will create an App component which has a prop queryParams which
 * is an object which contains page a number, search a string, and
 * showAll a boolean.
 *
 * @example
 * withQueryParams(App, { show: ['medium'] })
 *
 * Can also support arrays and convert to actual types as well.
 *
 * @param {String} options.url The url to be parsed
 * @param {Object} options.pathParams Consists of the matching key in the url to be replaced by the value.
 */
exports.withQueryParams = (Component, defaultQueryParams) => {
    class WithQueryParams extends react_1.default.Component {
        render() {
            const _a = this.props, { location } = _a, props = __rest(_a, ["location"]);
            const currentParams = location.search && query_string_1.parse(location.search);
            const queryParamsAsString = Object.assign({}, defaultQueryParams, currentParams);
            const typedQueryParams = convertQueryParamsToConcreteType(queryParamsAsString, defaultQueryParams, name);
            return react_1.default.createElement(Component, Object.assign({ queryParams: typedQueryParams }, props));
        }
    }
    if (process.env.NODE_ENV !== 'production') {
        WithQueryParams.displayName = wrapDisplayName_1.default(Component, 'withQueryParams');
    }
    return WithQueryParams;
};
